class KropffTabSelected:
    settings = "initial settings"
    high_tof = "high_tof"
    low_tof = "low_tof"
    bragg_peak = "bragg_peak"
    summary = "summary"


class FittingTabSelected:
    march_dollase = "march_dollase"
    kropff = "kropff"


class FittingKeys:
    x_axis = "xaxis"
    y_axis = "yaxis"
    transparency = "transparency"
    image_view_state = "image view state"
    image_view_histogram = "image view histogram"
    ui_accessed = "ui accessed"
    ui = "ui"
    row_index = "row_index"
    column_index = "column_index"
    bin_coordinates = "bin_coordinates"
    active = "active"
    lock = "lock"


selected_color = {"pen": (0, 0, 0, 30), "brush": (0, 255, 0, 150)}

lock_color = {"pen": (0, 0, 0, 30), "brush": (255, 0, 0, 240)}
