class TimeSpectraKeys:
    tof_array = "tof_array"
    lambda_array = "lambda_array"
    file_index_array = "file_index_array"
    file_name = "file_name"
    counts_array = "counts_array"


class CombineAlgorithm:
    """list of algorithm used to combine the folders"""

    mean = "mean"
    median = "median"
