class ParametersToDisplay:
    d = "d"
    strain_mapping = "strain_mapping"
    integrated_image = "integrated_image"


INTERPOLATION_METHODS = [
    "none",
    "nearest",
    "bilinear",
    "bicubic",
    "spline16",
    "spline36",
    "hanning",
    "hamming",
    "hermite",
    "kaiser",
    "quadric",
    "catrom",
    "gaussian",
    "bessel",
    "mitchell",
    "sinc",
    "lanczos",
]


DEFAULT_INTERPOLATION_INDEX = 0


CMAPS = ["viridis", "jet"]

DEFAULT_CMAP_INDEX = 0
